@@ -29,7 +29,9 @@
 #define HTTP_PARSER_RESPONSE_IMPL_HPP
 
 #include <algorithm>
+#include <istream>
 #include <sstream>
+#include <string>
 
 #include <websocketpp/http/parser.hpp>
 
@@ -37,7 +39,7 @@ namespace websocketpp {
 namespace http {
 namespace parser {
 
-inline size_t response::consume(const char *buf, size_t len) {
+inline size_t response::consume(char const * buf, size_t len) {
     if (m_state == DONE) {return 0;}
 
     if (m_state == BODY) {
@@ -170,34 +172,6 @@ inline size_t response::consume(std::istream & s) {
     return total;
 }
 
-inline bool response::parse_complete(std::istream& s) {
-    // parse a complete header (ie \r\n\r\n MUST be in the input stream)
-    std::string line;
-
-    // get status line
-    std::getline(s, line);
-
-    if (line[line.size()-1] == '\r') {
-        line.erase(line.end()-1);
-
-        std::stringstream   ss(line);
-        std::string         str_val;
-        int                 int_val;
-        char                char_val[256];
-
-        ss >> str_val;
-        set_version(str_val);
-
-        ss >> int_val;
-        ss.getline(char_val,256);
-        set_status(status_code::value(int_val),std::string(char_val));
-    } else {
-        return false;
-    }
-
-    return parse_headers(s);
-}
-
 inline std::string response::raw() const {
     // TODO: validation. Make sure all required fields have been set?
 
@@ -217,7 +191,7 @@ inline void response::set_status(status_code::value code) {
     m_status_msg = get_string(code);
 }
 
-inline void response::set_status(status_code::value code, const std::string&
+inline void response::set_status(status_code::value code, std::string const &
     msg)
 {
     // TODO: validation?
@@ -255,7 +229,7 @@ inline void response::process(std::string::iterator begin,
     set_status(status_code::value(code),std::string(cursor_end+1,end));
 }
 
-inline size_t response::process_body(const char *buf, size_t len) {
+inline size_t response::process_body(char const * buf, size_t len) {
     // If no content length was set then we read forever and never set m_ready
     if (m_read == 0) {
         //m_body.append(buf,len);
